@@ -1,8 +1,12 @@
+# frozen_string_literal: true
 require 'rubygems/test_case'
 require "rubygems/version"
 
 class TestGemVersion < Gem::TestCase
 
+  class V < ::Gem::Version
+  end
+
   def test_bump
     assert_bumped_version_equal "5.3", "5.2.4"
   end
@@ -23,14 +27,13 @@ class TestGemVersion < Gem::TestCase
     assert_bumped_version_equal "6", "5"
   end
 
-  # FIX: For "legacy reasons," any object that responds to +version+
-  # is returned unchanged. I'm not certain why.
+  # A Gem::Version is already a Gem::Version and therefore not transformed by
+  # Gem::Version.create
 
   def test_class_create
-    fake = Object.new
-    def fake.version; "1.0" end
+    real = Gem::Version.new(1.0)
 
-    assert_same  fake, Gem::Version.create(fake)
+    assert_same  real, Gem::Version.create(real)
     assert_nil   Gem::Version.create(nil)
     assert_equal v("5.1"), Gem::Version.create("5.1")
 
@@ -38,6 +41,13 @@ class TestGemVersion < Gem::TestCase
     assert_equal v('1.1'), Gem::Version.create(ver)
   end
 
+  def test_class_new_subclass
+    v1 = Gem::Version.new '1'
+    v2 = V.new '1'
+
+    refute_same v1, v2
+  end
+
   def test_eql_eh
     assert_version_eql "1.2",    "1.2"
     refute_version_eql "1.2",    "1.2.0"
@@ -72,7 +82,6 @@ class TestGemVersion < Gem::TestCase
       1.0\n2.0
       1..2
       1.2\ 3.4
-      1-2-3
     ].each do |bad|
       e = assert_raises ArgumentError, bad do
         Gem::Version.new bad
@@ -82,6 +91,12 @@ class TestGemVersion < Gem::TestCase
     end
   end
 
+  def test_empty_version
+    ["", "   ", " "].each do |empty|
+      assert_equal "0", Gem::Version.new(empty).version
+    end
+  end
+
   def test_prerelease
     assert_prerelease "1.2.0.a"
     assert_prerelease "2.9.b"
@@ -128,6 +143,23 @@ class TestGemVersion < Gem::TestCase
     assert_equal "5.2.4", v("5.2.4").to_s
   end
 
+  def test_semver
+    assert_less_than "1.0.0-alpha", "1.0.0-alpha.1"
+    assert_less_than "1.0.0-alpha.1", "1.0.0-beta.2"
+    assert_less_than "1.0.0-beta.2", "1.0.0-beta.11"
+    assert_less_than "1.0.0-beta.11", "1.0.0-rc.1"
+    assert_less_than "1.0.0-rc1", "1.0.0"
+    assert_less_than "1.0.0-1", "1"
+  end
+
+  # modifying the segments of a version should not affect the segments of the cached version object
+  def test_segments
+    v('9.8.7').segments[2] += 1
+
+    refute_version_equal "9.8.8", "9.8.7"
+    assert_equal         [9,8,7], v("9.8.7").segments
+  end
+
   # Asserts that +version+ is a prerelease.
 
   def assert_prerelease version
@@ -167,6 +199,12 @@ class TestGemVersion < Gem::TestCase
     assert second.eql?(first), "#{second} is eql? #{first}"
   end
 
+  def assert_less_than left, right
+    l = v(left)
+    r = v(right)
+    assert l < r, "#{left} not less than #{right}"
+  end
+
   # Refute the assumption that +version+ is a prerelease.
 
   def refute_prerelease version
